#!/usr/bin/env bash

#*****************************************************************************
# xtmvavg.sh version 1.0b
# 移動平均算出スクリプト
# Y.Hamuro
# 2004/05/07
#*****************************************************************************

function help {
cat >/dev/stderr <<EOF
--------------------------
xtmvavg.sh version 1.0b
--------------------------
概要) 移動平均を計算する。
書式) xtmvavg.sh -s 並べ換える項目名リスト -f 集計項目名 -n 移動する日数
                 -a 新項目名 [-i 入力ファイル名] [-o出力ファイル名]
                  [-T ワークディレクトリ名] [-V] [-h]
        -V : コマンドの完了メッセージを表示させる。
        -h : ヘルプの表示

例1) xtmvavg.sh -s 期 -f 金額 -n 3 -a 移動平均 -i input.xt -o output.xt
例2) xtmvavg.sh -s 期%r -f 数量,金額 -n 5 -a 移動平均 -i input2.xt -o output2.xt
例3) xtmvavg.sh -s 年,月 -f 金額 -n 4 -a 移動平均 <input3.xt >output3.xt

例1の入出力データ例
--------------
input.xtの内容
--------------
期 金額
1 10
2 20
3 30
4 40
5 50
--------------
output.xtの内容
--------------
期 金額 移動平均
1 10 *
2 20 *
3 30 20
4 40 30
5 50 40
--------------
期3 : (10+20+30)/3 = 20
期4 : (20+30+40)/3 = 30
期5 : (30+40+50)/3 = 40
EOF
exit 1
}

export mssQuiet=1

#パラメータのチェック＆セット
while getopts ":s:f:n:a:i:o:T:Vh" opt; do
  case $opt in
    s  ) SORT=$OPTARG ;;
    f  ) FLD=$OPTARG ;;
    n  ) MVCNT=$OPTARG ;;
    a  ) NEWFLD=$OPTARG ;;
    i  ) INPUT=$OPTARG ;;
    o  ) OUTPUT=$OPTARG ;;
    T  ) TMPD=$OPTARG ;;
    V  ) mssQuiet=0 ;;
    h  ) help ;;
    \? ) help ;;
  esac
done
shift $(($OPTIND -1 ))

if [ "$SORT" = "" ]; then
  echo "#MSG# $$ \"xtmvavg.sh\" \"option -s is mandatory\"" >/dev/stderr
  help; exit 1
fi
if [ "$FLD" = "" ]; then
  echo "#MSG# $$ \"xtmvavg.sh\" \"option -f is mandatory\"" >/dev/stderr
  help; exit 1
fi
if [ "$MVCNT" = "" ]; then
  echo "#MSG# $$ \"xtmvavg.sh\" \"option -n is mandatory\"" >/dev/stderr
  help; exit 1
fi
if [ "$NEWFLD" = "" ]; then
  echo "#MSG# $$ \"xtmvavg.sh\" \"option -a is mandatory\"" >/dev/stderr
  help; exit 1
fi
if [ "$INPUT" = "" ]; then
   INPUT=/dev/stdin
fi
if [ "$OUTPUT" = "" ]; then
 OUTPUT=/dev/stdout
fi
if [ "$TMPD" = "" ]; then
   TMPD=/tmp
fi

SORTFLD=${SORT%\%*}
SORTPAR=${SORT#*%}
if [ "$SORTFLD" = "$SORTPAR" ]; then
  SORTPAR=""
else
  SORTPAR="%$SORTPAR"
fi

TD=$TMPD/xxx$$

#=====================================================
# 【ステップ１】
# 入力ファイルの各行を移動数(-nの値)分duplicateし、
# 期内での連番項目(##mvSeq##)を作成する。
# この連番は期が新しくなる毎に開始番号をずらしておく。
#-----------------------------------------------------
# 期 金額  ($INPUT: 以下の込めんとはこの入力データを例にして説明している)
# 1 10
# 2 20
# 3 30
# 4 40
# 5 50
#  ↓
# 期 金額 ##mvSeq##  ($TD-xxbase)
# 1 10 1001
# 1 10 1002
# 1 10 1003
# 2 20 1002
# 2 20 1003
# 2 20 1004
# 3 30 1003
# 3 30 1004
# 3 30 1005
# 4 40 1004
# 4 40 1005
# 4 40 1006
# 5 50 1005
# 5 50 1006
# 5 50 1007
#=====================================================
xtnumber -s $SORTFLD$SORTPAR -S 1000 -a '##seqNo##' -i $INPUT |
xtsetchr -v $MVCNT -a days |
xtduprec -f days |
xtcal -k '##seqNo##' -c '$##seqNo##+keyLine()' -a '##mvSeq##' |
xtcut -r -f 'days,##seqNo##' -o $TD-xxbase

#=====================================================
# 【ステップ２】
# 次のステップ３で##mvSeq##をキーにして移動平均を求めるが
# その際、件数が移動数(-nの値)に満たないものは意味
# がない。
# そこで件数が移動数(-nの値)ある##mvSeq##を選んでおく。
#-----------------------------------------------------
# ##mvSeq## ##cnt##  ($TD-xx2)
# 1003 3
# 1004 3
# 1005 3
#=====================================================
xtcut   -f '##mvSeq##' -i $TD-xxbase | 
xtcount -k '##mvSeq##' -a '##cnt##' |
xtsel   -c '$##cnt##=='$MVCNT -o $TD-xx2

#=====================================================
# 【ステップ３】
# ##mvSeq##をキーにして移動平均を求める。
# その際、上記で求めた##mvSeq##の件数が移動数(-nの値)
# ある行のみを選ぶ。
#-----------------------------------------------------
# ##mvSeq## ##cnt##  ($TD-xx3)
# 1003 20
# 1004 30
# 1005 40
#=====================================================
xtcut    -f '##mvSeq##',$FLD -i $TD-xxbase |
xtagg    -k '##mvSeq##' -f $FLD:$NEWFLD -c avg |
xtcommon -k '##mvSeq##' -m $TD-xx2 -o $TD-xx3

#=====================================================
# 【ステップ４】
# ステップ１で求めたデータから、##mvSeqに対応する期を
# 選択し、ステップ３で求めた移動平均値を結合する。
#-----------------------------------------------------
# 期,金額,移動平均 ($OUTPUT)
# 1 10 *
# 2 20 *
# 3 30 20
# 4 40 30
# 5 50 40
#=====================================================
xtcut  -f '##mvSeq##',$SORTFLD,$FLD -i $TD-xxbase |
xtbest -k $SORTFLD -s '##mvSeq##' -R 1 |
xtjoin -n -k '##mvSeq##' -m $TD-xx3 -f $NEWFLD |
xtcut  -r -f '##mvSeq##' -o $OUTPUT

rm $TD-xx*
#===============================================================
